<?php

include_once 'DB.php';
include_once TURBA_BASE . '/lib/Solr/Service.php';

/**
 * Turba directory driver for obm address book.
 */
class Turba_Driver_obm extends Turba_Driver {

    /**
     * Handle for the database connection.
     *
     * @var DB
     */
    var $_db;

    /**
     * User id
     *
     * @var integer
     */
    var $_user_id;

    /**
     * User domain id
     *
     * @var integer
     */
    var $_domain_id;

    /**
     * Handle for the solr connection.
     *
     * @var Apache_Solr_Service
     */
    var $_solr;

    /**
     * list of user addressbooks id
     *
     * @var array
     */
    var $_addressbooks;

    /**
     * Solr fields
     *
     * @var array
     */
    var $_solrFields = array(
      'contact_id'         => 'id',
      'contact_firstname'  => 'firstname',
      'contact_lastname'   => 'lastname',
      'contact_middlename' => 'middlename',
      'contact_suffix'     => 'suffix',
      'contact_aka'        => 'aka',
      'contact_email'      => 'email',
      'contact_homephone'  => 'phone',
      'contact_workphone'  => 'phone',
      'contact_cellphone'  => 'phone',
      'contact_fax'        => 'fax',
      'contact_title'      => 'title',
      'contact_company'    => 'company',
      'contact_service'    => 'service',
      'contact_comment2'   => 'comment2',
    );

    /**
     * Initialize the obm driver.
     */
    function _init()
    {
        return true;
    }

    /**
     * Searches the address book with the given criteria and returns a
     * filtered list of results. If the criteria parameter is an empty array,
     * all records will be returned.
     *
     * @param array $criteria  Array containing the search criteria.
     * @param array $fields    List of fields to return.
     *
     * @return array  Hash containing the search results.
     */
    function _search($criteria, $fields)
    {
        $check = $this->query_init();
        if (is_a($check, 'PEAR_Error')) {
            Horde::logMessage($check, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $check;
        }

        $pattern = "domain:{$this->_domain_id}";
        $addressboods = $this->addressbooks();
        if (!empty($addressboods))
            $pattern .= " AND addressbookId:(".implode(' OR ',$addressboods).")";
        $cond = $this->buildSearchCriteria($criteria);
        if (!empty($cond))
            $pattern .= " AND $cond";

        $response = $this->_solr->search($pattern,0,1000);
        if($response->response->numFound <= 0)
            return array();
        //else

        $contact_ids = array();
        foreach ($response->response->docs as $doc) {
            array_push($contact_ids, $doc->id);
        }

        return $this->_read('contact_id', $contact_ids, $this->getContactOwner(), $fields);
    }

    /**
     * Reads the requested entries from the underlying source.
     *
     * @param string $key    The primary key field to use.
     * @param mixed $ids     The ids of the contacts to load.
     * @param string $owner  Only return contacts owned by this user.
     * @param array $fields  List of fields to return.
     *
     * @return array  Hash containing the search results.
     */
    function _read($key, $ids, $owner, $fields)
    {
        $check = $this->query_init();
        if (is_a($check, 'PEAR_Error')) {
            Horde::logMessage($check, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $check;
        }

        $contact_ids = implode(',',$ids);

//        $query = "
//            SELECT email_id, contactentity_contact_id AS contact_id, email_address FROM Email
//            INNER JOIN ContactEntity ON email_entity_id = contactentity_entity_id
//            WHERE contactentity_contact_id IN ({$contact_ids})
//            UNION SELECT email_id, contact_id, email_address FROM Email
//            INNER JOIN CompanyEntity ON email_entity_id = companyentity_entity_id
//            INNER JOIN Contact ON contact_company_id = companyentity_company_id
//            WHERE contact_id IN ({$contact_ids})
//            ORDER BY email_id";
//        $result = $this->_db->query($query);
//        if (is_a($result, 'PEAR_Error')) {
//            return $result;
//        }
//        $emails = array();
//        while ($row = $result->fetchRow(DB_FETCHMODE_ORDERED)) {
//            $contact_id = intval($row[1]);
//            //only one email address for each contact
//            if (!isset($emails[$contact_id]))
//                $emails[$contact_id] = $row[2];
//        }

        $owner = addslashes(Auth::getAuth());
        $query = "
            SELECT
                contact_id,
                '$owner' as owner,
                contact_lastname,
                contact_firstname,
                contact_middlename,
                contact_suffix,
                contact_aka,
                contact_title,
                COALESCE(company_name, contact_company) as contact_company,
                contact_service,
                contact_comment2,
                email_address as contact_email,
                wp.phone_number AS contact_workphone,
                hp.phone_number AS contact_homephone,
                cp.phone_number AS contact_cellphone,
                fax.phone_number AS contact_fax
            FROM Contact
            INNER JOIN ContactEntity ON contactentity_contact_id = contact_id
            LEFT JOIN Company ON contact_company_id = company_id
            LEFT JOIN Email ON email_entity_id = contactentity_entity_id
            LEFT JOIN Phone wp ON wp.phone_entity_id = contactentity_entity_id AND wp.phone_label LIKE 'WORK;VOICE%'
            LEFT JOIN Phone hp ON hp.phone_entity_id = contactentity_entity_id AND hp.phone_label LIKE 'HOME;VOICE%'
            LEFT JOIN Phone cp ON cp.phone_entity_id = contactentity_entity_id AND cp.phone_label LIKE 'CELL;VOICE%'
            LEFT JOIN Phone fax ON fax.phone_entity_id = contactentity_entity_id AND fax.phone_label LIKE 'WORK;FAX%'
            WHERE contact_id IN ($contact_ids)
              AND (email_id IN (SELECT MIN(email_id) FROM Email WHERE email_entity_id = contactentity_entity_id) OR email_id IS NULL)
              AND (wp.phone_id IN (SELECT MIN(phone_id) FROM Phone WHERE phone_entity_id = contactentity_entity_id AND phone_label LIKE 'WORK;VOICE%' ) OR wp.phone_id IS NULL)
              AND (hp.phone_id IN (SELECT MIN(phone_id) FROM Phone WHERE phone_entity_id = contactentity_entity_id AND phone_label LIKE 'HOME;VOICE%') OR hp.phone_id IS NULL)
              AND (cp.phone_id IN (SELECT MIN(phone_id) FROM Phone WHERE phone_entity_id = contactentity_entity_id AND phone_label LIKE 'CELL;VOICE%') OR cp.phone_id IS NULL)
              AND (fax.phone_id IN (SELECT MIN(phone_id) FROM Phone WHERE phone_entity_id = contactentity_entity_id AND phone_label LIKE 'WORK;FAX%') OR fax.phone_id IS NULL)";
        $result = $this->_db->query($query);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $results = array();
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
//            if (isset($emails[$row['contact_id']])) {
//                $row['contact_email'] = $emails[$row['contact_id']];
//            }
            array_push($results,$row);
        }
        return $results;
    }

    /**
     * _add is not supported for obm address books.
     */

    /**
     * _delete is not supported for obm address books.
     */

    /**
     * deleteAll is not supported for obm address books.
     */

    /**
     * _save is not supported for obm address books.
     */

    /**
     * Checks if the current user has the requested permissions on this
     * address book.
     *
     * @param integer $perm  The permissions to check against.
     *
     * @return boolean  True or False.
     */
    function hasPermission($perm)
    {
        return (parent::hasPermission($perm)) && ($perm & (PERMS_SHOW | PERMS_READ));
    }




    /**
     * Build dsn reading parameters from t_obm_conf global variable and init the db connection
     */
    function db_init()
    {
      if (empty($this->_db)) {
            $phptype = strtolower($this->_params['phptype']);
            $username = $this->_params['username'];
            $password = $this->_params['password'];
            $hostspec = $this->_params['hostspec'];
            $database_name = $this->_params['database_name'];
            $dsn = "$phptype://$username:$password@$hostspec/$database_name";
            if (empty($phptype) || empty($username) || empty($password) || empty($hostspec) || empty($database_name)) {
                return PEAR::raiseError(_("Missing parameters to connect to the database (dsn='$dsn')."));
            }

            $db = &DB::connect($dsn, array('persistent' => false));
            if (is_a($db, 'PEAR_Error')) {
                return $db;
            }
            $this->_db = $db;
        }
        return true;
    }

    /**
     * Read the user id and domain_id from database
     */
    function identify()
    {
        if (empty($this->_domain_id) || empty($this->_user_id)) {

            list($login, $domain) = split('@',Auth::getAuth());
            if (empty($login) || empty($domain)) {
                return PEAR::raiseError(_("Unable to guess current user or domain (login: $login, domain: $domain)."));
            }

            $query = "
                SELECT domain_id, userobm_id
                FROM Domain INNER JOIN UserObm ON userobm_domain_id=domain_id
                WHERE domain_name = '".addslashes($domain)."' AND userobm_login = '".addslashes($login)."'";
            $result = $this->_db->query($query);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }

            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if ((!$row) || empty($row['domain_id']) || empty($row['userobm_id'])) {
                return PEAR::raiseError(_("Unable to find user '$user' or domain '$domain' in the database."));
            }
            $this->_domain_id = intval($row['domain_id']);
            $this->_user_id = intval($row['userobm_id']);
        }
        return true;
    }

    /**
     * Initialize connection to the solr server
     */
    function solr_init()
    {
        if (!$this->_solr) {
            $query = "SELECT host_ip
                FROM Domain
                INNER JOIN DomainEntity ON domainentity_domain_id = domain_id
                INNER JOIN ServiceProperty ON serviceproperty_entity_id = domainentity_entity_id
                INNER JOIN Host ON host_id = CAST(serviceproperty_value AS INTEGER)
                WHERE serviceproperty_service = 'solr'
                  AND serviceproperty_property = 'contact'
                  AND (domain_id = {$this->_domain_id} OR domain_global = true)";
            $result = $this->_db->query($query);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
    
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if ((!$row) || empty($row['host_ip'])) {
                return PEAR::raiseError(_("Unable to find the solr host in the database."));
            }
    
            $solr_ip = $row['host_ip'];
            $this->_solr = new Apache_Solr_Service($solr_ip, '8080', "/solr/contact");
            if (!$this->_solr->ping()) {
                return PEAR::raiseError(_("Unable to connect to the solr host."));
            }
        }
        return true;
    }

    /**
     * Prepare the driver for query
     * doing this in _init will make some useless requests in a lot of case
     */
    function query_init()
    {
        $check = $this->db_init();
        if (is_a($check, 'PEAR_Error')) {
            return $check;
        }

        $check = $this->identify();
        if (is_a($check, 'PEAR_Error')) {
            return $check;
        }

        $check = $this->solr_init();
        if (is_a($check, 'PEAR_Error')) {
            return $check;
        }

        return true;
    }

    /**
     * Build search criteria
     */
    function buildSearchCriteria($criteria)
    {
        if (!is_array($criteria))
            return '';
        //else
        if (isset($criteria['field'])) {
            $field = $this->_solrFields[$criteria['field']];
            $value = strtolower(str_replace('*','',$criteria['test']));
            if (empty($field) || empty($value))
                return '';
            switch($criteria['op']) {
            case 'LIKE':
                return "$field:$value*";
            case '=':
            default:
                return "$field:$value";
            }
        }
        //else
        $cond = '';
        if (isset($criteria['AND']) && is_array($criteria['AND'])) {
            foreach ($criteria['AND'] as $conditions) {
                $newcond = $this->buildSearchCriteria($conditions);
                if (!empty($newcond)) {
                    $cond .= (empty($cond) ? '' : ' AND ') .$newcond;
                }
            }
        }
        if (isset($criteria['OR']) && is_array($criteria['OR'])) {
            foreach ($criteria['OR'] as $conditions) {
                $newcond = $this->buildSearchCriteria($conditions);
                if (!empty($newcond)) {
                    $cond .= (empty($cond) ? '' : ' OR ') .$newcond;
                }
            }
        }
        if (!empty($cond))
            $cond = "($cond)";
        return $cond;
    }

    /**
     * Get user addressbooks id
     */
    function addressbooks()
    {
        if (!is_array($this->_addressbooks)) {
            $query = "
                SELECT AddressBook.id FROM AddressBook
                WHERE AddressBook.owner = {$this->_user_id}
                UNION
                SELECT AddressBook.id
                FROM AddressBook
                INNER JOIN (
                    SELECT addressbookentity_addressbook_id
                    FROM UserEntity
                    INNER JOIN EntityRight ON userentity_entity_id = entityright_consumer_id
                    INNER JOIN AddressbookEntity ON addressbookentity_entity_id = entityright_entity_id
                    WHERE userentity_user_id = {$this->_user_id}
                      AND (entityright_access = 1 OR entityright_read = 1 OR entityright_write = 1 OR entityright_admin = 1)
                    UNION ALL
                    SELECT addressbookentity_addressbook_id
                    FROM EntityRight
                    INNER JOIN AddressbookEntity ON addressbookentity_entity_id = entityright_entity_id
                    INNER JOIN AddressBook as AllowedEntity ON addressbookentity_addressbook_id = AllowedEntity.id
                    WHERE entityright_consumer_id IS NULL AND AllowedEntity.domain_id = {$this->_domain_id}
                      AND (entityright_access = 1 OR entityright_read = 1 OR entityright_write = 1 OR entityright_admin = 1)
                    UNION ALL
                    SELECT addressbookentity_addressbook_id
                    FROM UserObm u1
                    INNER JOIN of_usergroup ON userobm_id = of_usergroup_user_id
                    INNER JOIN GroupEntity ON of_usergroup_group_id = groupentity_group_id
                    INNER JOIN EntityRight ON groupentity_entity_id = entityright_consumer_id
                    INNER JOIN AddressbookEntity ON addressbookentity_entity_id = entityright_entity_id
                    WHERE u1.userobm_id = {$this->_user_id}
                      AND (entityright_access = 1 OR entityright_read = 1 OR entityright_write = 1 OR entityright_admin = 1)
                ) AS Rights ON AddressBook.id = Rights.addressbookentity_addressbook_id
                WHERE AddressBook.domain_id = {$this->_domain_id}";
            $result = $this->_db->query($query);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }

            $this->_addressbooks = array();
            while ($row = $result->fetchRow(DB_FETCHMODE_ORDERED)) {
                if (!empty($row[0]))
                    $this->_addressbooks[] = intval($row[0]);
            }

            $this->_addressbooks = array_unique($this->_addressbooks);
        }
        return $this->_addressbooks;
    }
}
